# Provides an example of a self-navigating algorithm which you are under no obligation to use
from microprediction.writer import MicroWriter
from microprediction.reader import default_url
import muid, random, time
from microprediction.samplers import exponential_bootstrap
import pprint
import numpy as np
import os

class MicroCrawler(MicroWriter):
    """Typically you will want to override one or more of the following methods:

           candidate_streams()                          Return a list of stream names
           choose_horizon(name)                         Return int
           sample(lagged_values,lagged_times)           Creates samples from a vector of lagged value

       Good luck!
     """

    def __init__(self, write_key=None, base_url=None, verbose=True,stop_loss=500, min_budget=0., max_budget=5., min_lags = 25, sponsor_min=12, sleep_time=300):
        """  """
        super().__init__(base_url=base_url or default_url(), write_key=write_key )
        assert muid.difficulty(write_key) >= 8, "Invalid write_key for crawler. See www.muid.org to mine one. "
        assert not self.base_url[-1]=='/','Base url should not have trailing /'
        assert stop_loss>0,"Stop loss must be positive "
        self.verbose     = verbose
        self.sponsor_min = sponsor_min         # Only choose streams with sponsors at least this long
        self.sleep_time  = sleep_time          # Seconds to sleep between actions
        self.stop_loss   = stop_loss           # How much to lose before giving up on a stream
        self.min_budget  = min_budget          # Play with highly competitive algorithms?
        self.max_budget  = max_budget          # Play with highly competitive algorithms?
        self.min_lags    = min_lags            # Insist on historical data
        self.performance = self.get_performance()

    def candidate_streams(self):
        """ Should return a stream name, or None """
        budgets     = self.get_budgets()
        sponsors    = self.get_sponsors()
        # Stream criteria (combines with AND)
        not_too_dull         = [name for name, budget in budgets.items() if float(budget) >= self.min_budget]
        not_too_competitive  = [name for name, budget in budgets.items() if float(budget) <= self.max_budget ]
        well_sponsored       = [name for name, sponsor in sponsors.items() if len(sponsor.replace(' ','')) >= float(self.sponsor_min)]
        inclusion_criteria   = [not_too_dull, not_too_competitive, well_sponsored]
        # Choose at random
        candidates = list( set.intersection(*map(set,inclusion_criteria)))
        return candidates

    def choose_stream(self,exclude):
        candidates = self.candidate_streams()
        if exclude:
            candidates = [ c for c in candidates if not c in exclude ]
        if candidates:
            return random.choice(candidates)

    def cached_performance(self,name,delay):
        performance_key = os.path.splitext(name)[0] + self.sep() + str(delay)
        return self.performance.get(performance_key)

    def choose_horizon(self, name ):
        # Pick horizon where are aren't losing too much money yet, or withdraw
        delay_choices = list()
        for delay in self.delays:
            performance = self.cached_performance(name=name,delay=delay)
            if performance and float(performance)<-self.stop_loss:
                pass
            else:
                delay_choices.append(delay)
        if delay_choices:
            return random.choice(delay_choices)

    def withdraw_if_losing(self,name,delay):
        performance = self.cached_performance(name=name, delay=delay)
        if performance and float(performance)<-self.stop_loss:
            self.cancel(name=name, delays=[delay])
            if self.verbose:
                print( "Withdrawing from "+name+"::"+str(delay),flush=True )

    def sample(self, lagged_values, lagged_times=None):
        """ Should return a vector of scenarios of len self.num_predictions """
        return exponential_bootstrap(lagged=lagged_values,num=self.num_predictions, decay=0.01)

    def predict_and_submit(self, name, delay):
        """ Given a stream and horizon, try to submit predictions """
        lagged_values = self.get_lagged_values(name)
        lagged_times  = self.get_lagged_times(name)
        exec = 0
        if len(lagged_values or []) < self.min_lags:
            message = {'name': name, 'submitted': False, "reason": "Insufficient lags", "lagged_len": len(lagged_values)}
        else:
            scenario_values = self.sample(lagged_values=lagged_values,lagged_times=lagged_times)
            exec = self.submit(name=name, values=scenario_values, delay=delay)
            balance = self.get_balance()
            message = {'name': name, "submitted": True, 'delay': delay, "values": scenario_values[:2], "balance": balance,"exec":exec}
            if not exec:
                message.update({"submitted":False,"reason":"execution failure","confirms":self.get_confirms(), "errors": self.get_errors()})
        if self.verbose:
            pprint.pprint(message)
            print("", flush=True)
        return exec

    def run(self):
        """ Run until it doesn't seem to be working anywhere """
        blacklist = list()
        name = self.choose_stream(exclude=blacklist)
        while name:
            for delay in self.delays:
                self.withdraw_if_losing(name=name, delay=delay )
            delay = self.choose_horizon(name=name)
            if delay:
                self.predict_and_submit(name=name, delay=delay )
            else:
                blacklist.append(name)
            time.sleep(self.sleep_time)
            name = self.choose_stream(exclude=blacklist)
            if random.choice(range(10))==1:
                self.performance = self.get_performance()

        pprint.pprint(self.get_performance())
        print('Crawler is laying down to die ', flush=True )


    def initialization_checks(self):
        fake_lagged = list( np.random.randn(self.min_lags) )
        scenarios    = self.sample(fake_lagged)
        assert len(scenarios)==self.num_predictions, "This crawler will not work as the length of the "





import requests
import datetime
from pprint import  pprint
from apscheduler.schedulers.blocking import BlockingScheduler

from microprediction import MicroWriter
from microprediction.config_private import TRAFFIC_WRITE_KEY, BART_KEY

# Average delays for all BART train lines

def fetch_live_data(station):
    """ Given specific station (or "ALL") as input, returns the average delay in seconds """
    total_delay = 0
    lines = 0
    r = requests.get("http://api.bart.gov/api/etd.aspx?cmd=etd&orig="+station+"&key="+BART_KEY+"&json=y")
    if r.status_code==200:
        station_data = r.json()["root"]["station"]
        for station in station_data:
            for destination in station["etd"]:
                if len(destination["estimate"]) is not 0:
                    total_delay += int(destination["estimate"][0]["delay"])
                lines += 1
        return float(total_delay) / lines
    else:
        return None

def all_stations_delay():
    return fetch_live_data(station="ALL")



NAME = 'bart_delays.json'

initial_value = all_stations_delay()
print("Initial value is " + str(initial_value) + " seconds", flush=True)


def poll_and_send():
    """ Create stream of average delay in seconds """
    value = all_stations_delay()
    # res = mw.set(name=NAME,value=float(value))
    res = 0
    pprint({'average delay':value,"res":res})
    print('',flush=True)

def run():
    try:
        mw = MicroWriter(write_key=TRAFFIC_WRITE_KEY)
    except:
        raise Exception("You need to set the write key for this example to work")
    print('Starting scheduler',flush=True)
    scheduler = BlockingScheduler()
    scheduler.add_job(poll_and_send, 'interval', minutes=20)
    try:
        scheduler.start()
    except (KeyboardInterrupt, SystemExit):
        pass
    print('Stopping scheduler',flush=True)

if __name__=="__main__":
    run()




import requests
import datetime
from pprint import  pprint
from apscheduler.schedulers.blocking import BlockingScheduler

from microprediction import MicroWriter
from microprediction.config_private import TRAFFIC_WRITE_KEY

# Number of bikes available near large Manhattan hospitals
# longitude and latitude of rectangles within 500 feet of hospital
# citibike map: https://member.citibikenyc.com/map/
COORDS = [
    ((40.787880, -73.955862), (40.792001, -73.948389)),     # Mount Sinai - station_ids: 3345, 3299, 3363
    ((40.763573, -73.959175), (40.766666, -73.950179)),     # New York Presbytarian Upper East Side - station_ids: 3156, 3141
    ((40.735625, -73.979437), (40.743511, -73.970913)),     # Bellevue and NYU Langone - station_ids: 545, 3687, 174, 528, 536, 2012
    ((40.709229, -74.008624), (40.712033, -74.002787)),     # New York Presbytarian Lower Manhattan - station_ids: 224, 316
    ((40.783116, -73.949118), (40.787085, -73.939771)),     # Metropolitan Hospital Center - station_ids: 3309, 3318, 3338, 3351
]

def get_station_ids():
    """ Given COORDS, return all station_ids that fall within the rectangles """
    station_ids = []
    r = requests.get("https://feeds.citibikenyc.com/stations/stations.json")
    if r.status_code==200:
        data = r.json()
        for c1, c2 in COORDS:
            station_data = [ 
                d for d in data["stationBeanList"] \
                if d["latitude"] > c1[0] and d["latitude"] < c2[0] \
                and d["longitude"] > c1[1] and d["longitude"] < c2[1]
            ]
            for station in station_data:
                station_ids.append(int(station["id"]))
    else:
        raise Exception("Request Failed. Re-run.")
    return station_ids

def fetch_live_data(keys,field):
    """ Given list of station_ids as input, returns total number of bikes """
    r = requests.get("https://feeds.citibikenyc.com/stations/stations.json")
    if r.status_code==200:
        data = r.json()
        station_data = [ d for d in data["stationBeanList"] if int(d["id"]) in keys ]
        available_bikes = 0
        for station in station_data:
            available_bikes += station[field]
        return available_bikes
    else:
        return None

def madison_bikes_available():
    return fetch_live_data(keys=[456],field="availableBikes")

def hospital_bikes_available(station_ids):
    return fetch_live_data(keys=station_ids,field="availableBikes")



NAME = 'citibikes_near_manhattan_hospitals.json'

station_ids = get_station_ids()
initial_value = float(hospital_bikes_available(station_ids))
print("Initial value is " + str(initial_value), flush=True)


def poll_and_send():
    """ Create stream of citibike count """
    value = hospital_bikes_available(station_ids)
    # res = mw.set(name=NAME,value=float(value))
    res = 0
    pprint({'citibikes':value,"res":res})
    print('',flush=True)

def run():
    try:
        mw = MicroWriter(write_key=TRAFFIC_WRITE_KEY)
    except:
        raise Exception("You need to set the write key for this example to work")
    print('Starting scheduler',flush=True)
    scheduler = BlockingScheduler()
    scheduler.add_job(poll_and_send, 'interval', minutes=20)
    try:
        scheduler.start()
    except (KeyboardInterrupt, SystemExit):
        pass
    print('Stopping scheduler',flush=True)

if __name__=="__main__":
    run()

